package net.minecraft.src.forestry.api;

import java.util.ArrayList;
import java.util.HashMap;

import net.minecraft.src.ItemStack;

/**
 * Simple API class to add your own resources and recipes to Forestry
 * If you need something added/changed/fixed, drop me a PM on the Minecraft forums and I'll see what can be done.
 * You are free to include this API (forestry.api.*) in your own mod and redistribute and/or modify it at will.
 * Modifying will probably cause conflicts when Forestry is installed, so best to drop me a line if you need additions
 * or bugfixes.
 * @author SirSengir
 */
public class ForestryAPI {

	/**
	 * You can use the API from your own mod or write a simple plugin mod using this plugin interface. Adding
	 * a plugin here will have Forestry call the registered object during ModsLoaded(). See {@link IPlugin} for
	 * further details.
	 */
	public static ArrayList<IPlugin> plugins = new ArrayList<IPlugin>();
	
	
	/// RECIPE MANAGERS
	
	/**
	 * Allows you to add recipes to the carpenter. See {@link ICarpenterManager} for details.
	 */
	public static ICarpenterManager carpenterManager;
	/**
	 * Allows you to add recipes to the moistener. See {@link IMoistenerManager} for details.
	 */
	public static IMoistenerManager moistenerManager;
	/**
	 * Allows you to add recipes to the centrifuge. See {@link ICentrifugeManager} for details.
	 */
	public static ICentrifugeManager centrifugeManager;
	/**
	 * Allows you to add recipes to the still. See {@link IStillManager} for details.
	 */
	public static IStillManager stillManager;
	/**
	 * Allows you to add recipes to the bottler. See {@link IBottlerManager} for details.
	 */
	public static IBottlerManager bottlerManager;
	
	
	/// EXTENDING PLANTERS AND HARVESTERS
	
	/**
	 * Add additional germlings for the farm here.
	 */
	public static ArrayList<Germling> farmGermlings = new ArrayList<Germling>();
	/**
	 * Add additional yield for the combine here.
	 */
	public static ArrayList<Yield> combineYield = new ArrayList<Yield>();
	/**
	 * Add additional expected windfall for the combine here.
	 */
	public static ArrayList<ItemStack> combineWindfall = new ArrayList<ItemStack>();
	
	/// NOT YET USED
	public static ArrayList<Germling> arboretumGermlings = new ArrayList<Germling>();
	public static ArrayList<Yield> loggerYield = new ArrayList<Yield>();
	public static ArrayList<ItemStack> loggerWindfall = new ArrayList<ItemStack>();
	/// NOT YET USED
	public static ArrayList<Germling> melonfarmGermlings = new ArrayList<Germling>();
	public static ArrayList<Yield> melonharvesterYield = new ArrayList<Yield>();
	public static ArrayList<ItemStack> melonharvesterWindfall = new ArrayList<ItemStack>();
	/// NOT YET USED

	
	/// EXTENDING APIARIES
	
	/**
	 * Add flowers for use by apiaries here.
	 */
	public static ArrayList<ApiaryFlower> apiaryFlowers = new ArrayList<ApiaryFlower>();
	
	
	/// EXTENDING ENGINES
	
	/**
	 * Add new fuels for EngineBronze (= biogas engine) here
	 */
	public static HashMap<Integer, EngineBronzeFuel> bronzeEngineFuel = new HashMap<Integer, EngineBronzeFuel>();
	/**
	 * Add new fuels for EngineCopper (= peat-fired engine) here
	 */
	public static HashMap<Integer, EngineCopperFuel> copperEngineFuel = new HashMap<Integer, EngineCopperFuel>();
	
	
	/// ADDING MACHINE FUELS AND RESOURCES
	
	/**
	 * Add new fuels for the fermenter here (i.e. fertilizer)
	 */
	public static HashMap<Integer, FermenterFuel> fermenterFuel = new HashMap<Integer, FermenterFuel>();
	/**
	 * Add new resources for the fermenter here (i.e saplings)
	 */
	public static HashMap<Integer, FermenterResource> fermenterResource = new HashMap<Integer, FermenterResource>();
	/**
	 * Add new resources for the moistener here (i.e. wheat)
	 */
	public static HashMap<Integer, MoistenerResource> moistenerResource = new HashMap<Integer, MoistenerResource>();
	/**
	 * Add new substrates for the rainmaker here
	 */
	public static HashMap<Integer, RainSubstrate> rainSubstrate = new HashMap<Integer, RainSubstrate>();
	
	
	/// WRENCHES
	
	/**
	 * Register an item as wrench. Items with the passed itemid can be used to rotate {@BlockEngine}s.
	 * @param item ItemStack representing the wrench. Meta sensitive.
	 * @return
	 */
	public static boolean registerWrench(ItemStack item) {
		ItemStack put = new ItemStack(item.itemID, 0, item.getItemDamage());
		
		for(ItemStack wrench : wrenchs) {
			if(wrench.isItemEqual(item))
				return false;
		}
		
		wrenchs.add(put);
		return true;
	}
	
	
	/// LIQUID CONTAINERS
	
	/**
	 * Registers a {@link LiquidContainer} (bucket or bucket replacement) for use as a generic liquid container in Forestry's machines. 
	 * @param container
	 * @return
	 */
	public static boolean registerLiquidContainer(LiquidContainer container) {
		// Key already taken, skip adding this container
		if(liquidContainersByFullId.containsKey(container.filled.itemID))
			return false;
		
		liquidContainersByFullId.put(container.filled.itemID, container);
		return true;
	}
	
	/**
	 * Registers a {@link LiquidContainer} (bucket or bucket replacement) for use as a lava container in Forestry's machines. 
	 * @param container
	 * @return
	 */
	public static boolean registerLavaContainer(LiquidContainer container) {
		// Key already taken, skip adding this container
		if(lavaContainersByFullId.containsKey(container.filled.itemID))
			return false;
		
		lavaContainersByFullId.put(container.filled.itemID, container);
		if(container.empty != null)
			lavaContainersByEmptyId.put(container.empty.itemID, container);
		return true;
	}
	
	/**
	 * Registers a {@link LiquidContainer} (bucket or bucket replacement) for use as a water container in Forestry's machines. 
	 * @param container
	 * @return
	 */
	public static boolean registerWaterContainer(LiquidContainer container) {
		// Key already taken, skip adding this container
		if(waterContainersByFullId.containsKey(container.filled.itemID))
			return false;
		
		waterContainersByFullId.put(container.filled.itemID, container);
		if(container.empty != null)
			waterContainersByEmptyId.put(container.empty.itemID, container);
		return true;
	}
	
	/**
	 * Registers a {@link LiquidContainer} (bucket or bucket replacement) for use as a biomass container in Forestry's machines. 
	 * @param container
	 * @return
	 */
	public static boolean registerBiomassContainer(LiquidContainer container) {
		// Key already taken, skip adding this container
		if(biomassContainersByFullId.containsKey(container.filled.itemID))
			return false;
		
		biomassContainersByFullId.put(container.filled.itemID, container);
		if(container.empty != null)
			biomassContainersByEmptyId.put(container.empty.itemID, container);
		return true;
	}
	
	/**
	 * Registers a {@link LiquidContainer} (bucket or bucket replacement) for use as a biofuel container in Forestry's machines. 
	 * @param container
	 * @return
	 */
	public static boolean registerBiofuelContainer(LiquidContainer container) {
		// Key already taken, skip adding this container
		if(biofuelContainersByFullId.containsKey(container.filled.itemID))
			return false;
		
		biofuelContainersByFullId.put(container.filled.itemID, container);
		if(container.empty != null)
			biofuelContainersByEmptyId.put(container.empty.itemID, container);
		return true;
	}
	
	// EVERYTHING BELOW IS FOR INTERNAL USE ONLY - SHOULD BE SELF EXPLANATORY
	// (and should probably be moved somewhere else in the code, but it's harmless and I'm lazy.)
	private static ArrayList<ItemStack> wrenchs = new ArrayList<ItemStack>();	
	
	public static HashMap<Integer, LiquidContainer> liquidContainersByFullId = new HashMap<Integer, LiquidContainer>();

	public static HashMap<Integer, LiquidContainer> lavaContainersByFullId = new HashMap<Integer, LiquidContainer>();
	private static HashMap<Integer, LiquidContainer> lavaContainersByEmptyId = new HashMap<Integer, LiquidContainer>();
	public static HashMap<Integer, LiquidContainer> waterContainersByFullId = new HashMap<Integer, LiquidContainer>();
	private static HashMap<Integer, LiquidContainer> waterContainersByEmptyId = new HashMap<Integer, LiquidContainer>();
	public static HashMap<Integer, LiquidContainer> biomassContainersByFullId = new HashMap<Integer, LiquidContainer>();
	private static HashMap<Integer, LiquidContainer> biomassContainersByEmptyId = new HashMap<Integer, LiquidContainer>();
	public static HashMap<Integer, LiquidContainer> biofuelContainersByFullId = new HashMap<Integer, LiquidContainer>();
	private static HashMap<Integer, LiquidContainer> biofuelContainersByEmptyId = new HashMap<Integer, LiquidContainer>();
	
	// GENERIC
	public static boolean hasLiquidContainerByFullId(int id) {
		return liquidContainersByFullId.containsKey(id);
	}
	public static LiquidContainer getLiquidContainerByFullId(int id) {
		return liquidContainersByFullId.get(id);
	}
	
	// LAVA
	public static boolean hasLavaContainerByFullId(int id) {
		return lavaContainersByFullId.containsKey(id);
	}
	public static boolean hasLavaContainerByEmptyId(int id) {
		return lavaContainersByEmptyId.containsKey(id);
	}
	public static LiquidContainer getLavaContainerByFullId(int id) {
		return lavaContainersByFullId.get(id);
	}
	public static LiquidContainer getLavaContainerByEmptyId(int id) {
		return lavaContainersByEmptyId.get(id);
	}
	
	// WATER
	public static boolean hasWaterContainerByFullId(int id) {
		return waterContainersByFullId.containsKey(id);
	}
	public static boolean hasWaterContainerByEmptyId(int id) {
		return waterContainersByEmptyId.containsKey(id);
	}
	public static LiquidContainer getWaterContainerByFullId(int id) {
		return waterContainersByFullId.get(id);
	}
	public static LiquidContainer getWaterContainerByEmptyId(int id) {
		return waterContainersByEmptyId.get(id);
	}
	
	// BIOMASS
	public static boolean hasBiomassContainerByFullId(int id) {
		return biomassContainersByFullId.containsKey(id);
	}
	public static boolean hasBiomassContainerByEmptyId(int id) {
		return biomassContainersByEmptyId.containsKey(id);
	}
	public static LiquidContainer getBiomassContainerByFullId(int id) {
		return biomassContainersByFullId.get(id);
	}
	public static LiquidContainer getBiomassContainerByEmptyId(int id) {
		return biomassContainersByEmptyId.get(id);
	}
	
	// BIOFUEL
	public static boolean hasBiofuelContainerByFullId(int id) {
		return biofuelContainersByFullId.containsKey(id);
	}
	public static boolean hasBiofuelContainerByEmptyId(int id) {
		return biofuelContainersByEmptyId.containsKey(id);
	}
	public static LiquidContainer getBiofuelContainerByFullId(int id) {
		return biofuelContainersByFullId.get(id);
	}
	public static LiquidContainer getBiofuelContainerByEmptyId(int id) {
		return biofuelContainersByEmptyId.get(id);
	}
	
	public static boolean isWrench(ItemStack item) {
		for(ItemStack wrench : wrenchs) {
			if(wrench.isItemEqual(item))
				return true;
		}
		return false;
	}
}
