package net.minecraft.src.forestry.api;

import net.minecraft.src.ItemStack;
import net.minecraft.src.World;

/**
 * Represents the yield from a farm or arboretum. Used by {@link Harvester} to determine what to harvest, what to store and whether or not to drop any fruits (windfall) when harvesting.
 */
public class Yield {
	/**
	 * Ripe product block.
	 */
	public final ItemStack ripe;
	/**
	 * ItemStack that is actually added to the harvester's inventory on plucking the ripe product.
	 */
	public final ItemStack plucked;
	/**
	 * The ItemStack to drop as windfall.
	 */
	public ItemStack windfall;
	/**
	 * Block to create at the place of {@link ripe} after harvesting.
	 */
	public ItemStack waste;
	
	private boolean metaSensitive;
	/**
	 * Whether an exact metaRequired is required. If false a value equal or larger than {@link metaRequired} will be required.
	 */
	private boolean metaExact;
	/**
	 * The meta data to require.
	 */
	private int metaRequired;
	
	/**
	 * Stem to leave. If this is set to 2, only the third block will be harvested.
	 */
	private int requiredStem = 0; 
	
	public Yield(ItemStack ripe, ItemStack plucked)
	{
		this.ripe = ripe;
		this.plucked = plucked;
	}
	
	public void configureWindfall(ItemStack windfall) {
		this.windfall = windfall;
	}
	
	public void configureWaste(ItemStack waste) {
		this.waste = waste;
	}
	
	public void configureStem(ItemStack stem, int i) {
		this.requiredStem = i;
	}
	
	public void configureMeta(int metaRequired, boolean metaExact) {
		this.metaSensitive = true;
		this.metaRequired = metaRequired;
		this.metaExact = metaExact;
	}
	
	/// INTERNAL USE BELOW
	
	public boolean createsWindfall() {
		return windfall != null;
	}
	
	public boolean leavesWaste() {
		return waste != null;
	}
	
	/**
	 * Internal use
	 * @param meta
	 * @return
	 */
	public boolean validateMeta(int meta)
	{
		if(!metaSensitive)
			return true;
		
		if(metaExact)
		{
			if(metaRequired == meta)
				return true;
		}
		else
		{
			if(meta >= metaRequired)
				return true;
		}
		
		return false;
	}
	
	public boolean requiresStem() {
		return requiredStem > 0;
	}
	
	/**
	 * Check whether we have the required amount of stems. Not metadata aware!
	 * @param world
	 * @param pos
	 * @return
	 */
	public boolean validateStem(World world, int i, int j, int k) {
		
		if(requiredStem <= 0)
			return true;
		
		for(int y = 1; y <= requiredStem; y++) {
			int blockid = world.getBlockId(i, j-y, k);
			if(blockid != ripe.itemID)
				return false;
		}
		
		return true;
	}
}
